'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var Schema = _interopDefault(require('async-validator'));
var _clone = _interopDefault(require('lodash/clone'));
var _setWith = _interopDefault(require('lodash/setWith'));
var _get = _interopDefault(require('lodash/get'));
var tslib_1 = require('tslib');
var React = require('react');
var classNames = _interopDefault(require('classnames'));

function getRules(prop, rules, trigger) {
    if (!rules || !rules[prop]) {
        return [];
    }
    var result = Array.isArray(rules[prop]) ? rules[prop] : [rules[prop]];
    if (!trigger) {
        return result;
    }
    return result.filter(function (rule) {
        return !rule.trigger || rule.trigger.indexOf(trigger) !== -1;
    });
}
function isRequired(prop, rules) {
    var data = getRules(prop, rules);
    if (data.length) {
        for (var index = 0; index < data.length; index++) {
            var rule = data[index];
            if (rule.required) {
                return true;
            }
        }
    }
    return false;
}
function getValidKeys(formValue, keys) {
    var allKeys = Object.keys(formValue);
    if (!keys) {
        return allKeys;
    }
    if (!Array.isArray(keys)) {
        keys = [keys];
    }
    return keys.filter(function (v) { return allKeys.indexOf(v) !== -1; });
}
function find(arr, fn) {
    for (var _i = 0, arr_1 = arr; _i < arr_1.length; _i++) {
        var item = arr_1[_i];
        if (fn(item)) {
            return item;
        }
    }
    return undefined;
}

function baseValidate(formValue, keys, rules, trigger) {
    var descriptor = {};
    var data = {};
    var validKeys = getValidKeys(formValue, keys);
    validKeys.forEach(function (key) {
        var rule = getRules(key, rules, trigger);
        var value = _get(formValue, key);
        data[key] = value;
        if (rule.length) {
            descriptor[key] = rule;
        }
    });
    if (Object.keys(descriptor).length === 0) {
        return Promise.resolve(data);
    }
    return new Promise(function (resolve, reject) {
        var validator = new Schema(descriptor);
        validator.validate(data, function (errors) {
            if (errors) {
                var error_1 = {};
                validKeys.forEach(function (key) {
                    error_1[key] = find(errors, function (o) { return o.field === key; });
                });
                reject(error_1);
            }
            else {
                resolve(data);
            }
        });
    });
}
function init(initialValue) {
    return { value: tslib_1.__assign({}, initialValue), error: {} };
}
function reducer(state, action) {
    var value = state.value, error = state.error;
    var type = action.type, payload = action.payload;
    switch (type) {
        case 'setValue':
            Object.keys(payload).forEach(function (key) {
                _setWith(value, key, payload[key], _clone);
            });
            return { value: value, error: error };
        case 'setError':
            Object.keys(payload).forEach(function (key) {
                _setWith(error, key, payload[key], _clone);
            });
            return { value: value, error: error };
        case 'reset':
            return init(payload);
        default:
            throw new Error();
    }
}
function useFormx(initialValue, rules) {
    if (initialValue === void 0) { initialValue = {}; }
    var _a = React.useReducer(reducer, initialValue, init), formState = _a[0], dispatch = _a[1];
    var setFieldsValue = React.useCallback(function (payload) {
        dispatch({ type: 'setValue', payload: payload });
    }, []);
    var setFieldsError = React.useCallback(function (payload) {
        dispatch({ type: 'setError', payload: payload });
    }, []);
    var dispatchField = React.useCallback(function (payload) {
        var _a;
        var type = payload.type, prop = payload.prop, value = payload.value, error = payload.error;
        var data = (_a = {}, _a[prop] = value, _a);
        if (type === 'change') {
            setFieldsValue(data);
        }
        baseValidate(data, prop, rules, type)
            .then(function () {
            var _a;
            if (error) {
                setFieldsError((_a = {}, _a[prop] = undefined, _a));
            }
        })
            .catch(function (err) {
            setFieldsError(err);
        });
    }, [rules]);
    var resetFields = React.useCallback(function () {
        dispatch({ type: 'reset', payload: initialValue });
    }, []);
    var getField = function (prop) {
        return [_get(formState.value, prop), _get(formState.error, prop)];
    };
    var validate = function (keys) {
        return baseValidate(formState.value, keys, rules).catch(function (err) {
            setFieldsError(err);
            throw err;
        });
    };
    var bindFormx = function (prop) {
        var required = isRequired(prop, rules);
        var _a = getField(prop), value = _a[0], error = _a[1];
        return {
            required: required,
            value: value,
            error: error,
            prop: prop,
            key: prop,
            dispatch: dispatchField
        };
    };
    return tslib_1.__assign({}, formState, { bindFormx: bindFormx,
        validate: validate,
        setFieldsValue: setFieldsValue,
        setFieldsError: setFieldsError,
        resetFields: resetFields,
        getField: getField });
}

var FormxContext = React.createContext({
    labelPosition: 'right',
    labelSuffix: ':'
});

var Provider = FormxContext.Provider;
var useMemo = React.useMemo, useCallback = React.useCallback;
function Formx(props) {
    var _a = props.labelPosition, labelPosition = _a === void 0 ? 'right' : _a, _b = props.labelSuffix, labelSuffix = _b === void 0 ? ':' : _b, labelWidth = props.labelWidth, disabled = props.disabled, className = props.className, restProps = tslib_1.__rest(props, ["labelPosition", "labelSuffix", "labelWidth", "disabled", "className"]);
    var memoized = useMemo(function () {
        return { labelPosition: labelPosition, labelWidth: labelWidth, labelSuffix: labelSuffix, disabled: disabled };
    }, [labelPosition, labelWidth, labelSuffix, disabled]);
    var formClasses = classNames('easy-formx', "easy-formx--label-" + labelPosition, className);
    var prevent = useCallback(function (e) { return e.preventDefault(); }, []);
    return (React.createElement(Provider, { value: memoized },
        React.createElement("form", tslib_1.__assign({ onSubmit: prevent, className: formClasses }, restProps))));
}

var useContext = React.useContext;
function isObject(obj) {
    return obj !== null && typeof obj === 'object';
}
function getValue(event) {
    if (!isObject(event) || !event.target) {
        return event;
    }
    var target = event.target;
    var value = target.type === 'checkbox' ? target.checked : target.value;
    return value;
}
function getStyle(labelPosition, labelWidth) {
    var label = {};
    var content = {};
    if (labelPosition === 'left' || labelPosition === 'right') {
        label.textAlign = labelPosition;
        if (labelWidth) {
            label.width = labelWidth;
            content.marginLeft = labelWidth;
        }
    }
    return { label: label, content: content };
}
function FormxItem(props) {
    var prop = props.prop, value = props.value, error = props.error, required = props.required, dispatch = props.dispatch, 
    // above from `bindFormx
    label = props.label, labelWidth = props.labelWidth, labelPosition = props.labelPosition, labelStyle = props.labelStyle, _a = props.trigger, trigger = _a === void 0 ? 'onChange' : _a, _b = props.valuePropName, valuePropName = _b === void 0 ? 'value' : _b, children = props.children, className = props.className, rest = tslib_1.__rest(props, ["prop", "value", "error", "required", "dispatch", "label", "labelWidth", "labelPosition", "labelStyle", "trigger", "valuePropName", "children", "className"]);
    var _c = useContext(FormxContext), globalLabelPosition = _c.labelPosition, globalLabelWidth = _c.labelWidth, labelSuffix = _c.labelSuffix, disabled = _c.disabled;
    var message = error && error.message;
    var handleChange = function (event) {
        var value = getValue(event);
        if (prop && dispatch) {
            dispatch({ type: 'change', prop: prop, value: value, error: error });
        }
    };
    var getChangeFn = function (originalChanage) {
        return function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            if (typeof originalChanage === 'function') {
                originalChanage.apply(void 0, args);
            }
            handleChange(args[0]);
        };
    };
    var handleBlur = function () {
        if (prop && dispatch) {
            dispatch({ type: 'blur', prop: prop, value: value, error: error });
        }
    };
    var itemClasses = classNames('easy-formx-item', className, {
        'has-error': error,
        'easy-formx-item--with-help': message
    });
    var labelClasses = classNames('easy-formx-item__label', {
        'is-required': required
    });
    var style = getStyle(labelPosition || globalLabelPosition, labelWidth || globalLabelWidth);
    var items = children;
    if (prop) {
        items = React.Children.map(children, function (child, index) {
            var _a;
            if (index === 0 && React.isValidElement(child)) {
                return React.cloneElement(child, (_a = {
                        disabled: child.props.disabled || disabled
                    },
                    _a[valuePropName] = value,
                    _a[trigger] = getChangeFn(child.props[trigger]),
                    _a));
            }
            return child;
        });
    }
    return (React.createElement("div", tslib_1.__assign({}, rest, { className: itemClasses, onBlur: handleBlur }),
        label && (React.createElement("label", { className: labelClasses, style: tslib_1.__assign({}, style.label, labelStyle) },
            label,
            React.createElement("span", { className: "esay-formx-item__suffix" }, labelSuffix))),
        React.createElement("div", { className: "easy-formx-item__content", style: style.content },
            items,
            message && React.createElement("div", { className: "easy-formx-item__error" }, message))));
}

exports.useFormx = useFormx;
exports.Formx = Formx;
exports.FormxItem = FormxItem;
//# sourceMappingURL=index.js.map
